// ROOT Libraries
#include <TEventList.h>
#include <TH1D.h>
#include <TH2D.h>
#include <THStack.h>
// Standard Libraries
#include <map>
#include <sstream>
#include <vector>
//Other
#include "cut.h"
#include "event.h"


#ifndef CREATEZPLOT_H
#define CREATEZPLOT_H

using std::map;
using std::string;
using std::stringstream;
using std::vector;

class DrawingOptions {
	public:
		enum TYPE { MARK, FILL, HATCH };
		DrawingOptions(const char * fn);
		~DrawingOptions() {};
		void readOptions(const char * fileName);
		void applyOptions(TH1D * histo, string name);
		string getLegendOptions(const string & name);
		string getDrawOptions(const string & name, bool same);
		TYPE getHistogramType(const string & name);
		unsigned getStack(const string & name);
	private:
		struct HistOpt {
			string name_;
			TYPE type_;
			unsigned style_;
			float size_;
			unsigned color_;
			string leg_;
			string draw_;
			unsigned stack_;
		};
		vector<HistOpt> options_;


};

typedef pair<TH1D *, string> HS;

void makePlots (vector<HS> histos, DrawingOptions drawOp, string outputDir);
double findHistosNonZeroMinimum( const vector<TH1D *> histos );
double findHistosMaximum( const vector<TH1D *> histos, THStack & stack);

#endif
