// Standard Libraries
#include <cmath>
#include <iostream>
#include <sstream>
#include <string>
#include <utility>
// Other
#include "event.h"

using std::cin;
using std::cout;
using std::endl;
using std::string;
using std::pair;

#ifndef CUT_H
#define CUT_H

class Cut {
	private:
		double value;
	protected:
		string name;
	public:
		Cut (double val, const string & nm = "no name") : value (val), name (nm) {
			SetName (name);
		};
		virtual ~Cut() {};
		virtual string GetBareName() const {
			return name;
		};
		virtual string GetName() const {
			std::stringstream temp;
			temp << name << " " << GetValue();
			return temp.str();
		}
		virtual void SetName (const string & nm) {
			name = nm;
		}
		virtual bool operator() (const event &) const = 0;
		virtual double GetValue() const {
			return value;
		};
		virtual void SetValue (double val) {
			value = val;
		}
};

class DoubleCut : public Cut {
	private:
		double value2;
		virtual double GetValue() const {
			return Cut::GetValue();
		}
	public:
		DoubleCut (double val1, double val2, const string & nm = "no name") : Cut (val1, nm), value2 (val2) {};
		virtual bool operator() (const event & ev) const = 0;
		virtual double GetValueX() const {
			return GetValue();
		}
		virtual void SetValueX (double val) {
			SetValue (val);
		}
		virtual double GetValueY() const {
			return value2;
		}
		virtual void SetValueY (double val) {
			value2 = val;
		}
		virtual pair<double, double> GetPairValue() const {
			return pair<double, double> (GetValueX(), GetValueY());
		}
		virtual void SetValues (double val1, double val2) {
			SetValueX (val1);
			SetValueY (val2);
		}
};

class ObjSelector {
	public:
		ObjSelector() {};
		virtual ~ObjSelector() {};
		virtual bool operator() (const event & ev, unsigned index) const = 0;
};
/*
class JetSelector : public ObjSelector {
	private:
		double pt_min;
		double eta_max;
		const unsigned max_index;
	public:
		JetSelector (double pt, double eta) : pt_min (pt), eta_max (eta), max_index (25) {};
		double GetPtMin() const {
			return pt_min;
		}
		double GetEtaMax() const {
			return eta_max;
		}
		virtual bool operator() (const event & ev, unsigned index) const {
			if (index > max_index) {
				std::cout << "Warning: Index bigger than " << max_index << "!" << std::endl;
				return false;
			}
			if (ev.RECOJET_PT[index] > GetPtMin() && std::fabs (ev.RECOJET_ETA[index]) < GetEtaMax()
					&& ev.RECOJET_DELTAR1LEP[index] > 0.5 && ev.RECOJET_DELTAR2LEP[index] > 0.5)
				return true;
			return false;
		}
};
*/
class No_Cut : public Cut {
	public:
		No_Cut() : Cut (0, "No cut") {};
		No_Cut (const string & nm) : Cut (0, nm) {};
		virtual bool operator() (const event & ev) const {
/*
			cout << "Run: " << ev.getUnsignedVariableValue("Run");
			cout << " Lumi: " << ev.getUnsignedVariableValue("LumiSection");
			cout << " Event: " << ev.getUnsignedVariableValue("Event") << endl;
			cout << "PT: ";
			for (int i = 0; i < 5; i++)
				cout << ev.getDouble5ArrayValue("RECO_PT", i) << " ";
			cout << endl;
			cin.get();
 */
			return true;
		}
		virtual string GetName() const {
			return Cut::name;
		}
};

class Met_Cut : public Cut {
	public:
		Met_Cut (double val, const string & nm = "MET") : Cut (val, nm) {};
		virtual bool operator() (const event &) const;
};

class Met_max_Cut : public Cut {
	public:
		Met_max_Cut (double val, const string & nm = "MET") : Cut (val, nm) {};
		virtual bool operator() (const event &) const;
};

class Met_Sign_Cut : public Cut {
	public:
		Met_Sign_Cut (double val, const string & nm = "MET_Sign") : Cut (val, nm) {};
		virtual bool operator() (const event &) const;
};

class Zmass_min_Cut : public Cut {
	public:
		Zmass_min_Cut (double val, const string & nm = "M_Z_min") : Cut (val, nm) {};
		virtual bool operator() (const event &) const;
};

class Zmass_max_Cut : public Cut {
	public:
		Zmass_max_Cut (double val, const string & nm = "M_Z_max") : Cut (val, nm) {};
		virtual bool operator() (const event &) const;
};

class Zpt_min_Cut : public Cut {
	public:
		Zpt_min_Cut (double val, const string & nm = "PT_Z_min") : Cut (val, nm) {};
		virtual bool operator() (const event &) const;
};

class Zpt_max_Cut : public Cut {
	public:
		Zpt_max_Cut (double val, const string & nm = "PT_Z_max") : Cut (val, nm) {};
		virtual bool operator() (const event &) const;
};

class ZMet_acop_Cut : public Cut {
	public:
		ZMet_acop_Cut (double val, const string & nm = "Z-MET_Acop") : Cut (val, nm) {};
		virtual bool operator() (const event &) const;
};

class JetVeto_Cut : public Cut {
	public:
		JetVeto_Cut (double val, const string & nm = "Jet_Veto") : Cut (val, nm) {};
		virtual bool operator() (const event &) const;
};

class dPhiMET2Lep_Cut : public Cut {
	public:
		dPhiMET2Lep_Cut (double val, const string & nm = "dPMET2L") : Cut (val, nm) {};
		virtual bool operator() (const event &) const;
};

class Btag_Cut : public Cut {
	public:
		Btag_Cut (double val, const string & nm = "B_Tag") : Cut (val, nm) {};
		virtual bool operator() (const event &) const;
};

class Nlep_Cut : public Cut {
	public:
		Nlep_Cut (double val, const string & nm = "N_lep") : Cut (val, nm) {};
		virtual bool operator() (const event &) const;
};

class Isol_Cut : public Cut {
	public:
		Isol_Cut (double val, const string & nm = "Isol") : Cut (val, nm) {};
		virtual bool operator() (const event &) const;
};

class ZMetPt_Cut : public Cut {
	public:
		ZMetPt_Cut (double val, const string & nm = "PT_ZMET") : Cut (val, nm) {};
		virtual bool operator() (const event &) const;
};

class Hmass_min_Cut : public Cut {
	private:
		int index;
	public:
		Hmass_min_Cut (double val, const string & nm = "M_H_min") : Cut (val, nm), index (0) {};
		virtual bool operator() (const event &) const;
};

class Hmass_max_Cut : public Cut {
	private:
		int index;
	public:
		Hmass_max_Cut (double val, const string & nm = "M_H_max") : Cut (val, nm), index (0) {};
		virtual bool operator() (const event &) const;
};

class PtMinPtMax_Cut : public Cut {
	public:
		PtMinPtMax_Cut (double val, const string & nm = "PtMinMax") : Cut (val, nm) {};
		virtual bool operator() (const event &) const;
};

class ProjMet_Cut : public Cut {
	public:
		ProjMet_Cut (double val, const string & nm = "ProjMET") : Cut (val, nm) {};
		virtual bool operator() (const event &) const;
};

class DPhiLep_Cut : public Cut {
	public:
		DPhiLep_Cut (double val, const string & nm = "DPhiLep") : Cut (val, nm) {};
		virtual bool operator() (const event &) const;
};

class LepPt_Cut : public Cut {
	public:
		LepPt_Cut (double val, const string & nm = "LepPt") : Cut (val, nm) {};
		virtual bool operator() (const event &) const;
};

class TrackIso_Cut : public Cut {
	public:
		TrackIso_Cut (double val, const string & nm = "TrackIso") : Cut (val, nm) {};
		virtual bool operator() (const event &) const;
};

class Nvtx_Cut : public Cut {
	public:
		Nvtx_Cut (double val, const string & nm = "N_Vtx") : Cut (val, nm) {};
		virtual bool operator() (const event &) const;
};

#endif // CUT_H
