// ROOT Libraries
#include <TBranch.h>
#include <TCanvas.h>
#include <TH2.h>
#include <TLeaf.h>
#include <TObjArray.h>
#include <TStyle.h>
// Standard Libraries
#include <iostream>
#include <cstdlib>
#include <string>
#include <utility>
#include <vector>
// Other
#include "event.h"

using std::cout;
using std::cin;
using std::endl;
using std::pair;
using std::string;
using std::vector;

event::event( TTree *tree )
{
	TObjArray * treeLeaves = tree->GetListOfLeaves();
	int nEntries = treeLeaves->GetEntries();
	TObjArray::Iterator_t iter( treeLeaves );
	for( int i = 0; i < nEntries; ++i ) {
		iter.Next();
		TLeaf * leafPointer = dynamic_cast<TLeaf *>( *iter );
		if( leafPointer ) {
			if ( leafPointer->GetTypeName() == string("UInt_t") && leafPointer->GetLen() == 1 ) {
				pair<string, unsigned *> temp(leafPointer->GetBranch()->GetName(), new unsigned);
				tree->SetBranchAddress( leafPointer->GetBranch()->GetName(), temp.second );
				unsignedTypes.push_back(temp);
			} else if ( leafPointer->GetTypeName() == string("Int_t") && leafPointer->GetLen() == 1 ) {
				pair<string, int *> temp(leafPointer->GetBranch()->GetName(), new int);
				tree->SetBranchAddress( leafPointer->GetBranch()->GetName(), temp.second );
				intTypes.push_back(temp);
			} else if ( leafPointer->GetTypeName() == string("Double_t") && leafPointer->GetLen() == 1 ) {
				pair<string, double *> temp(leafPointer->GetBranch()->GetName(), new double);
				tree->SetBranchAddress( leafPointer->GetBranch()->GetName(), temp.second );
				doubleTypes.push_back(temp);
			} else if ( leafPointer->GetTypeName() == string("UInt_t") && leafPointer->GetLen() == 10 ) {
				pair<string, unsigned *> temp(leafPointer->GetBranch()->GetName(), new unsigned[10]);
				tree->SetBranchAddress( leafPointer->GetBranch()->GetName(), temp.second );
				unsigned10Types.push_back(temp);
			} else if ( leafPointer->GetTypeName() == string("Double_t") && leafPointer->GetLen() == 5 ) {
				pair<string, double *> temp(leafPointer->GetBranch()->GetName(), new double[5]);
				tree->SetBranchAddress( leafPointer->GetBranch()->GetName(), temp.second );
				double5Types.push_back(temp);
			} else if ( leafPointer->GetTypeName() == string("Double_t") && leafPointer->GetLen() == 3 ) {
				pair<string, double *> temp(leafPointer->GetBranch()->GetName(), new double[3]);
				tree->SetBranchAddress( leafPointer->GetBranch()->GetName(), temp.second );
				double3Types.push_back(temp);
			} else if ( leafPointer->GetTypeName() == string("Int_t") && leafPointer->GetLen() == 5 ) {
				pair<string, int *> temp(leafPointer->GetBranch()->GetName(), new int[5]);
				tree->SetBranchAddress( leafPointer->GetBranch()->GetName(), temp.second );
				int5Types.push_back(temp);
			} else if ( leafPointer->GetTypeName() == string("Double_t") && leafPointer->GetLen() == 25 ) {
				pair<string, double *> temp(leafPointer->GetBranch()->GetName(), new double[25]);
				tree->SetBranchAddress( leafPointer->GetBranch()->GetName(), temp.second );
				double25Types.push_back(temp);
			} else {
				cout << "ERROR: Don't know what to do with this type: " << leafPointer->GetTypeName() << '[' << leafPointer->GetLen() << "]!" << endl;
				exit (EXIT_FAILURE);
			}
		}
	}
}

event::~event() {
	for (unsigned i = 0; i < unsignedTypes.size(); ++i)
		delete unsignedTypes[i].second;
	for (unsigned i = 0; i < intTypes.size(); ++i)
		delete intTypes[i].second;
	for (unsigned i = 0; i < doubleTypes.size(); ++i)
		delete doubleTypes[i].second;
	for (unsigned i = 0; i < unsigned10Types.size(); ++i)
		delete [] unsigned10Types[i].second;
	for (unsigned i = 0; i < double5Types.size(); ++i)
		delete [] double5Types[i].second;
	for (unsigned i = 0; i < double3Types.size(); ++i)
		delete [] double3Types[i].second;
	for (unsigned i = 0; i < int5Types.size(); ++i)
		delete [] int5Types[i].second;
	for (unsigned i = 0; i < double25Types.size(); ++i)
		delete [] double25Types[i].second;
}

unsigned event::getUnsignedVariableValue(const string & name) const {
	for (unsigned i = 0; i < unsignedTypes.size(); ++i) {
		if ( unsignedTypes[i].first == name )
			return *(unsignedTypes[i].second);
	}
	cout << "ERROR: Couldn't find " << name << " variable!" << endl;
	exit( EXIT_FAILURE );
}
		
unsigned event::getUnsigned10ArrayValue(const string & name, unsigned n) const {
	if ( n > 9 ) {
		cout << "ERROR: Index out of range!" << endl;
		exit( EXIT_FAILURE );
	}
	for (unsigned i = 0; i < unsigned10Types.size(); ++i) {
		if ( unsigned10Types[i].first == name )
			return *(unsigned10Types[i].second + n);
	}
	cout << "ERROR: Couldn't find " << name << " variable!" << endl;
	exit( EXIT_FAILURE );
}

const unsigned * event::getUnsignedVariableAdr(const string & name) const {
	for (unsigned i = 0; i < unsignedTypes.size(); ++i) {
		if ( unsignedTypes[i].first == name )
			return unsignedTypes[i].second;
	}
	cout << "ERROR: Couldn't find " << name << " variable!" << endl;
	exit( EXIT_FAILURE );
}

const unsigned * event::getUnsigned10ArrayAdr(const string & name, unsigned n) const {
	if ( n > 9 ) {
		cout << "ERROR: Index out of range!" << endl;
		exit( EXIT_FAILURE );
	}
	for (unsigned i = 0; i < unsigned10Types.size(); ++i) {
		if ( unsigned10Types[i].first == name )
			return unsigned10Types[i].second + n;
	}
	cout << "ERROR: Couldn't find " << name << " variable!" << endl;
	exit( EXIT_FAILURE );
}

const int * event::getIntVariableAdr(const string & name) const {
	for (unsigned i = 0; i < intTypes.size(); ++i) {
		if ( intTypes[i].first == name )
			return intTypes[i].second;
	}
	cout << "ERROR: Couldn't find " << name << " variable!" << endl;
	exit( EXIT_FAILURE );
}

const double * event::getDoubleVariableAdr(const string & name) const {
	for (unsigned i = 0; i < doubleTypes.size(); ++i) {
		if ( doubleTypes[i].first == name )
			return doubleTypes[i].second;
	}
	cout << "ERROR: Couldn't find " << name << " variable!" << endl;
	exit( EXIT_FAILURE );
}

double event::getDouble5ArrayValue(const string & name, unsigned n) const {
	if ( n > 4 ) {
		cout << "ERROR: Index out of range!" << endl;
		exit( EXIT_FAILURE );
	}
	for (unsigned i = 0; i < double5Types.size(); ++i) {
		if ( double5Types[i].first == name )
			return *(double5Types[i].second + n);
	}
	cout << "ERROR: Couldn't find " << name << " variable!" << endl;
	exit( EXIT_FAILURE );
}

const double * event::getDouble5ArrayAdr(const string & name, unsigned n) const {
	if ( n > 4 ) {
		cout << "ERROR: Index out of range!" << endl;
		exit( EXIT_FAILURE );
	}
	for (unsigned i = 0; i < double5Types.size(); ++i) {
		if ( double5Types[i].first == name )
			return double5Types[i].second + n;
	}
	cout << "ERROR: Couldn't find " << name << " variable!" << endl;
	exit( EXIT_FAILURE );
}

const double * event::getDouble3ArrayAdr(const string & name, unsigned n) const {
	if ( n > 2 ) {
		cout << "ERROR: Index out of range!" << endl;
		exit( EXIT_FAILURE );
	}
	for (unsigned i = 0; i < double3Types.size(); ++i) {
		if ( double3Types[i].first == name )
			return double3Types[i].second + n;
	}
	cout << "ERROR: Couldn't find " << name << " variable!" << endl;
	exit( EXIT_FAILURE );
}

bool event::findVariable(const string & name, VARTYPE & vt) const {
	for (unsigned i = 0; i < unsignedTypes.size(); ++i) {
		if ( name == unsignedTypes[i].first) {
			vt = UNSIGN;
			return true;
		}
	}
	for (unsigned i = 0; i < intTypes.size(); ++i) {
		if ( name == intTypes[i].first) {
			vt = INT;
			return true;
		}
	}
	for (unsigned i = 0; i < doubleTypes.size(); ++i) {
		if ( name == doubleTypes[i].first) {
			vt = DOUBLE;
			return true;
		}
	}
	for (unsigned i = 0; i < unsigned10Types.size(); ++i) {
		if ( name == unsigned10Types[i].first) {
			vt = UNSIGN10;
			return true;
		}
	}
	for (unsigned i = 0; i < double5Types.size(); ++i) {
		if ( name == double5Types[i].first) {
			vt = DOUBLE5;
			return true;
		}
	}
	for (unsigned i = 0; i < double3Types.size(); ++i) {
		if ( name == double3Types[i].first) {
			vt = DOUBLE3;
			return true;
		}
	}
	for (unsigned i = 0; i < int5Types.size(); ++i) {
		if ( name == int5Types[i].first) {
			vt = INT5;
			return true;
		}
	}
	for (unsigned i = 0; i < double25Types.size(); ++i) {
		if ( name == double25Types[i].first) {
			vt = DOUBLE25;
			return true;
		}
	}
	return false;
}
