#ifndef event_h
#define event_h

#include <TROOT.h>
#include <TTree.h>
#include <TFile.h>
#include <string>
#include <utility>
#include <vector>

using std::pair;
using std::string;
using std::vector;

class event
{
	public :
		enum VARTYPE { UNSIGN, INT, DOUBLE, UNSIGN10, DOUBLE5, DOUBLE3, INT5, DOUBLE25 };
		event(TTree *tree);
		~event();
		unsigned getUnsignedVariableValue(const string & name) const;
		unsigned getUnsigned10ArrayValue(const string & name, unsigned n) const;
		const unsigned * getUnsignedVariableAdr(const string & name) const;
		const unsigned * getUnsigned10ArrayAdr(const string & name, unsigned n) const;
		const int * getIntVariableAdr(const string & name) const;
		const double * getDoubleVariableAdr(const string & name) const;
		double getDouble5ArrayValue(const string & name, unsigned n) const;
		const double * getDouble5ArrayAdr(const string & name, unsigned n) const;
		const double * getDouble3ArrayAdr(const string & name, unsigned n) const;
		bool findVariable(const string & name, VARTYPE & vt) const;
	private :
		vector<pair<string, unsigned *> > unsignedTypes;
		vector<pair<string, int *> > intTypes;
		vector<pair<string, double *> > doubleTypes;
		vector<pair<string, unsigned *> > unsigned10Types;
		vector<pair<string, double *> > double5Types;
		vector<pair<string, double *> > double3Types;
		vector<pair<string, int *> > int5Types;
		vector<pair<string, double *> > double25Types;

};

#endif
