#include "histograms1d.h"
#include <fstream>
#include <iostream>
#include <cstdlib>
#include <string>
#include <sstream>
#include <iomanip>

using std::cin;
using std::cout;
using std::endl;
using std::string;
using std::stringstream;
using std::fixed;
using std::flush;
using std::setprecision;

Histograms1D::Histograms1D(const char * fn) {
	readHistograms(fn);
}

Histograms1D::Histograms1D(const Histograms1D & histos) {
	name_ = histos.name_;
	histos1D.clear();
	for (unsigned i = 0; i < histos.histos1D.size(); ++i) {
		TH1D * tempH = new TH1D(*histos.histos1D[i]);
		tempH->SetDirectory( 0 );
		histos1D.push_back(tempH);
	}
}

Histograms1D::~Histograms1D() {
	for (unsigned i = 0; i < histos1D.size(); ++i) {
		delete histos1D[i];
	}
}

void Histograms1D::readHistograms(const char* fileName)
{
	ifstream input( fileName );
	if( !input.is_open() ) {
		cout << "ERROR: Can't open file with samples (" << fileName << ")!" << endl;
		exit( EXIT_FAILURE );
	}
	while( !input.eof() ) {
		string temp;
		getline( input, temp );
		if( !temp.size() || temp[0] == '\n' || temp[0] == '#' ) {
			continue;
		}
		unsigned pos = temp.find('"');
		temp.erase(0, pos + 1);
		pos = temp.find('"');
		string name = temp.substr(0, pos);
		temp.erase(0, pos + 1);
		pos = temp.find('"');
		temp.erase(0, pos + 1);
		pos = temp.find('"');
		string title = temp.substr(0, pos);
		temp.erase(0, pos + 1);
		stringstream otherPar(temp);
		int nBins;
		double lBin, hBin;
		otherPar >> nBins >> lBin >> hBin;
		TH1D * tempH = new TH1D(name.c_str(), title.c_str(), nBins, lBin, hBin);
		tempH->SetDirectory( 0 );
		histos1D.push_back(tempH);
	}
}

void Histograms1D::fillHistograms(const Samples::sample & sample) {
	name_ = sample.GetName();
	TFile file( sample.GetPath().c_str() );
	TTree * tree = ( TTree * ) file.Get( "HZZ4LeptonsAnalysis" );
	event ev( tree );
	
	vector<HistoFiller> hFillers;
	for (unsigned i = 0; i < histos1D.size(); ++i) {
		HistoFiller temp(histos1D[i]->GetName(), ev);
		hFillers.push_back(temp);
	}

	if( sample.GetEventList().GetN() == 0 ) {
		cout << "WARNING: " << sample.GetName() << " : Empty list!" << endl;
		return;
	}
	const TEventList & in_list = sample.GetEventList();
	unsigned long nentries = in_list.GetN();
	cout << "Filling histograms for " << sample.GetName() << "..." << endl;
	for( unsigned long i = 0; i < nentries; i++ ) {
		if(( i % 1000 ) == 0 ) {
			cout << string( 10, '\b' );
			cout << fixed << setprecision( 0 ) << i * 100.0 / nentries << "% done" << flush;
		}
		unsigned long en = in_list.GetEntry( i );
		tree->GetEntry( en );
		for (unsigned j = 0; j < histos1D.size(); ++j) {
			// cout << histos1D[j]->GetName() << " : " << setprecision(5) << hFillers[j].getValue() << endl;
			histos1D[j]->Fill( hFillers[j].getValue() );
		}
	}
	cout << string( 10, '\b' );
	cout << "100% done" << endl;
	return;
}

void Histograms1D::scaleHistograms(double sf) {
	for (unsigned i = 0; i < histos1D.size(); ++i)
		histos1D[i]->Scale( sf );
}

TH1D * Histograms1D::getHisto(unsigned n) {
	if ( n >= histos1D.size() ) {
		cout << "ERROR: Index out of range!" << endl;
		exit( EXIT_FAILURE );
	} else
		return histos1D[n];
}

HistoFiller::HistoFiller(string hName, const event & ev) {
	event::VARTYPE vt;
	unsigned objN;
	char lastChar = hName[hName.size() - 1];
	bool found = false;
	if (lastChar == '0' || lastChar == '1' || lastChar == '2' || lastChar == '3' || lastChar == '4') {
		stringstream number;
		number << lastChar;
		number >> objN;
		hName.erase(hName.size() - 2, 2);
		found = ev.findVariable(hName, vt);
	}
	if (found) {
		if (vt == event::UNSIGN10) {
			type_ = UNSIGNED;	
			valueU_ = ev.getUnsigned10ArrayAdr(hName, objN);
		} else if (vt == event::DOUBLE5) {
			type_ = DOUBLE;
			valueD_ = ev.getDouble5ArrayAdr(hName, objN);
		} else if ( vt == event::DOUBLE3 ) {
			type_ = DOUBLE;
			valueD_ = ev.getDouble3ArrayAdr(hName, objN);
		} else {
			cout << "ERROR: This type shouln't be here!" << endl;
			exit( EXIT_FAILURE );
		}
	} else if ( ev.findVariable(hName, vt) ) {
		if (vt == event::UNSIGN) {
			type_ = UNSIGNED;	
			valueU_ = ev.getUnsignedVariableAdr(hName);
		} else if (vt == event::INT) {
			type_ = INT;
			valueI_ = ev.getIntVariableAdr(hName);
		} else if (vt == event::DOUBLE) {
			type_ = DOUBLE;
			valueD_ = ev.getDoubleVariableAdr(hName);
		}
	} else {
		cout << "ERROR: Don't know what to do with this variable: " << hName << "!" << endl;
		exit( EXIT_FAILURE );
	}
}

double HistoFiller::getValue() {
	if (type_ == UNSIGNED)
		return (double) (*valueU_);
	else if (type_ == INT)
		return (double) (*valueI_);
	else
		return (*valueD_);
}
