// ROOT Libraries
#include "TCanvas.h"
#include "TH1D.h"
#include "TLegend.h"
#include "TStyle.h"
// Standard Libraries
#include <cmath>
#include <ctime>
#include <fstream>
#include <iomanip>
#include <iostream>
#include <sstream>
#include <string>
// Other
#include "createplot.h"
#include "cut.h"
#include "histograms1d.h"
#include "samples.h"

using std::cin;
using std::cout;
using std::endl;
using std::ios;
using std::ofstream;
using std::setw;
using std::stringstream;

const string workingDir = "/home/zablocki/CMS/HZZ2l2nu/";

int main()
{
	DrawingOptions drawOp("/home/zablocki/CMS/HZZ2l2nu/plotOptions.txt");

	ofstream testOut("/home/zablocki/test.txt");
	Samples allSamples("/home/zablocki/CMS/HZZ2l2nu/samples_all.txt");
	allSamples.selectEvents();
//	allSamples.setWorkingLumi(1000);
	allSamples.printNumbersHeading(testOut);
	allSamples.printScaleFactors(testOut);
	allSamples.printRawNumbers(testOut);
	allSamples.printNumbers(testOut, "Preselection");

	Histograms1D dataHis("/home/zablocki/CMS/HZZ2l2nu/histos1D.txt");
	dataHis.fillHistograms(allSamples.getDataSample());
	
	Histograms1D signalHis("/home/zablocki/CMS/HZZ2l2nu/histos1D.txt");
	signalHis.fillHistograms(allSamples.getSignalSample());
	signalHis.scaleHistograms(allSamples.getSignalScaleFactor() );

	vector<Histograms1D> backgroundHis;
	for (unsigned i = 0; i < allSamples.getNbackgroundSamples(); ++i) {
		Histograms1D tempBg("/home/zablocki/CMS/HZZ2l2nu/histos1D.txt");
		tempBg.fillHistograms( allSamples.getBackgroundSample(i) );
		tempBg.scaleHistograms( allSamples.getBackgroundScaleFactor(i) );
		backgroundHis.push_back(tempBg);
	}
	for (unsigned i = 0; i < dataHis.getNhistos(); ++i) {
		vector<HS> tempHistos;
		tempHistos.push_back( HS( dataHis.getHisto(i), dataHis.getName() ) );
		tempHistos.push_back( HS( signalHis.getHisto(i), signalHis.getName() ) );
		for (unsigned j = 0; j < backgroundHis.size(); ++j) {
			tempHistos.push_back( HS( backgroundHis[j].getHisto(i), backgroundHis[j].getName() ) );
		}
		makePlots(tempHistos, drawOp, "/home/zablocki/testPlots/");
	}	

	cout << "DONE!" << endl;

}
