// ROOT Libraries
#include <TEventList.h>
// Standard Libraries
#include <string>
#include <vector>
// Other
#include "cut.h"

#ifndef SAMPLES_H
#define SAMPLES_H

using std::string;
using std::vector;

class Samples
{
	private:
		enum SampleType {DATA, SIGNAL, BACKGROUND};
	public:
		Samples(const char * fileName);
		~Samples();
		void readSamples( const char * fileName );
		void printNumbers( ostream & output, const string & rowName );
		void printNumbersHeading( ostream & output );
		void printScaleFactors( ostream & output );
		void printRawNumbers( ostream & output );
		void setWorkingSignalSample( unsigned n );
		void setWorkingDataSample( unsigned n );
		double significance();
		void setWorkingLumi(double l);
		void selectEvents(const Cut & cut = No_Cut(), bool readListFromTree = true);
		unsigned getNdataSamples() {
			return dataSamples.size();
		}
		unsigned getNsignalSamples() {
			return signalSamples.size();
		}
		unsigned getNbackgroundSamples() {
			return backgroundSamples.size();
		}
		class sample {
			public:
				sample( string n, string p, double xs, unsigned int ne, SampleType st ) : name( n ), path( p ), xsection( xs ), n_events( ne ),
				sampleType( st ) {
					lumi = n_events / xsection;
					n_selected = n_events;
				};
				sample( string n, string p, double l, SampleType st ) : name( n ), path( p ), xsection( 0 ), n_events( 0 ),
				sampleType( st ), lumi( l ) {
					n_selected = n_events;
				};
				string GetName() const {
					return name;
				}
				unsigned GetNselected() const {
					return n_selected;
				}
				unsigned GetNevents() const{
					return n_events;
				}
				double GetLumi() const {
					return lumi;
				}
				double GetCrossSection() const {
					return xsection;
				}
				string GetPath() const {
					return path;
				}
				TEventList & GetEventList() {
					return sel_events;
				}
				const TEventList & GetEventList() const {
					return sel_events;
				}
				void SetNselected(unsigned n) {
					n_selected = n;
				}
			private:
				string name;
				string path;
				double xsection;
				unsigned int n_events;
				SampleType sampleType;
				double lumi;
				unsigned int n_selected;
				TEventList sel_events;
		};
		const Samples::sample & getDataSample() const {
			return dataSamples[workingDataSample];
		}
		const Samples::sample & getSignalSample() const {
			return signalSamples[workingSignalSample];
		}
		const Samples::sample & getBackgroundSample( unsigned n );
		double getSignalScaleFactor();
		double getBackgroundScaleFactor(unsigned n);
	private:
		vector<sample> dataSamples;
		vector<sample> signalSamples;
		vector<sample> backgroundSamples;
		int workingSignalSample;
		int workingDataSample;
		double workingLumi;
		void selectSampleEvents(sample & samp, const Cut & cut, bool readListFromTree);
};

#endif // SAMPLES_H
